diff --git a/paranoia/paranoia.c b/paranoia/paranoia.c
index 07b0322..075b937 100644
--- a/paranoia/paranoia.c
+++ b/paranoia/paranoia.c
@@ -863,6 +863,10 @@ static long i_iterate_stage2(cdrom_paranoia *p,v_fragment *v,
      */
     sort_setup(i,fv(v),&fb(v),fs(v),fbv,fev);
 
+    long min_matchbegin = -1;
+    long min_matchend = -1;
+    long min_offset = -1;
+
     for(j=searchbegin;j<searchend;j+=23){
 
       /* Skip past silence in the root.  If there are just a few silent
@@ -894,27 +898,38 @@ static long i_iterate_stage2(cdrom_paranoia *p,v_fragment *v,
        */
       if(try_sort_sync(p,i,NULL,rc(root),j,
 		       &matchbegin,&matchend,&offset,callback)){
-	
-	/* If we found a matching run, we return the results of our match.
-	 *
-	 * Note that we flip the sign of (offset) because try_sort_sync()
-	 * returns it in terms of the fragment (i.e. what we add
-	 * to the fragment's position to yield the corresponding position
-	 * in the root), but here we consider the root to be canonical,
-	 * and so our returned "offset" reflects how the fragment is offset
-	 * from the root.
-	 *
-	 * E.g.: If the fragment's sample 10 corresponds to root's 12,
-	 * try_sort_sync() would return 2.  But since root is canonical,
-	 * we say that the fragment is off by -2.
-	 */
-	r->begin=matchbegin;
-	r->end=matchend;
-	r->offset=-offset;
-	if(offset)if(callback)(*callback)(r->begin,PARANOIA_CB_FIXUP_EDGE);
-	return(1);
+        if(abs(offset) < abs(min_offset) || min_offset == -1) {
+          min_matchbegin = matchbegin;
+          min_matchend = matchend;
+          min_offset = offset;
+          if(min_offset >= 0) {
+            /* We will never find a smaller offset by continuing */
+            break;
+          }
+        }
       }
     }
+
+    /* If we found a matching run, we return the results of our match.
+     *
+     * Note that we flip the sign of (offset) because try_sort_sync()
+     * returns it in terms of the fragment (i.e. what we add
+     * to the fragment's position to yield the corresponding position
+     * in the root), but here we consider the root to be canonical,
+     * and so our returned "offset" reflects how the fragment is offset
+     * from the root.
+     *
+     * E.g.: If the fragment's sample 10 corresponds to root's 12,
+     * try_sort_sync() would return 2.  But since root is canonical,
+     * we say that the fragment is off by -2.
+     */
+    if(min_offset != -1) {
+      r->begin=min_matchbegin;
+      r->end=min_matchend;
+      r->offset=-min_offset;
+      if(offset)if(callback)(*callback)(r->begin,PARANOIA_CB_FIXUP_EDGE);
+      return(1);
+    }
   }
   
   return(0);

